#!/usr/bin/perl -W

#   IMPORTANT: READ THIS IN ITS ENTIRETY. YOU ARE THE FINAL AUTHORITY IN DETERMINING
#   WHAT HAPPENS ON YOUR COMPUTER. YOU SHOULD READ ALL DOCUMENTATION PERTAINING TO
#   THE INSTALLATION AND USE OF ALL SOFTWARE YOU INSTALL AND USE AND SHOULD MAKE 
#   REGULAR AND COMPLETE BACKUPS OF YOUR IMPORTANT DATA.
#
#   TickerMonster - Download and parsing script for retrieving
#   stock market ticker lists of American, NASDAQ, and New York Stock Exchange
#   listed stocks. 
#
#   -------------------------LEGAL INFO--------------------------------------------
#
#   Notice: XMLWorks has been acquired by 10 Squared Corporation.
#   TickerMonster and other free software projects developed by XMLWorks will
#   continue to be offered by 10 Squared Corporation at no charge. 
#
#   This script is Copyright (C) 2004  10 Squared Corporation All Rights Reserved
#
#   This is version 0.01
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
#   --------------------------------USAGE INFO-------------------------------------
#
#   USAGE: tickerMonster
#
#   Run tickerMonster from the command line. Windows users may need to use
#   perl tickerMonster
#
#   Three files are produced: amex.ticker, ny.ticker, and nasdaq.ticker
#
#   -----------------------------SUPPORT INFO--------------------------------------
#
#   Email: tickermonster@xmlworks.com
#
#   Since this is free software, no support is offered. However, suggestions for improvements
#   to the program are always welcome. If you are using the program and find it useful,
#   please drop us a line sometime. We would enjoy hearing from you.
#
#   Snail Mail: 10 Squared Corporation
#               PO Box 941124
#               Atlanta, GA 31141-0124
#               USA 
#   Web: http://www.xmlworks.com/
#
#   Custom modifications to this script or other custom Internet programming
#   in Perl are also available.

# ----- DO NOT EDIT CODE BELOW THIS LINE UNLESS YOU KNOW WHAT YOU ARE DOING ! ----- 

sub get {
    $mozilla = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Hotbar 4.4.6.0; .NET CLR 1.0.3705';
    use LWP::UserAgent;
    my $ua = LWP::UserAgent->new();
    $ua->agent($mozilla);
    my $url = shift(@_);
    my $file = shift(@_);
    use HTTP::Request;
    use HTTP::Response;
    print "Retrieving $url\n";
    my $req = HTTP::Request->new(GET => $url);
    my $response = $ua->request($req);
    if ($response->is_error()) {
      printf "%s\n", $response->status_line;
    }
    else {
	my $content = $response->content();
	print "Retrieved $url\n";
	open RESPONSE, ">$file" || warn "$t Cannot open $file for writing $!";
	print RESPONSE $content;
	close RESPONSE;
    }
}
# SET THE URLS FOR THE EXCHANGES
$nasdaq = 'http://www.nasdaq.com//asp/symbols.asp?exchange=Q&start=0';
$amex = 'http://www.nasdaq.com//asp/symbols.asp?exchange=1&start=0';
$ny = 'http://www.nasdaq.com//asp/symbols.asp?exchange=N&start=0';

# GET THE RAW DATA FOR EACH EXCHANGE
get $nasdaq, 'nasdaq.raw';
get $amex, 'amex.raw';
get $ny, 'ny.raw';

# PROCESS THE RAW DATA

# The data is in csv format, but contains embedded carriage returns, so things get a little messy...
@exchange = ('nasdaq', 'ny', 'amex');
for $exchange (@exchange) {
	open IN, "$exchange.raw" or die $!;
	$cr = $/;
	undef $/;
	$data = (<IN>);
	close IN;
	$data =~ s/","/|/gs;
	$/ = $cr;
	$data =~ s/\|/$cr/gs;
	open TEMP, ">$exchange.~mp";
	print TEMP $data;
	close TEMP;
	open IN, "$exchange.~mp";
	open OUT, ">$exchange.ticker";
	$count = 0;
	while (<IN>) {
		chomp;
		if (/^[A-Z]+$/ && length $_ < 7) {
			print OUT "$_\n";
			$count++;
		}
		elsif (/^[A-Z]+\/[A-Z]/ && length $_ < 7) {
			s/\///;
			print OUT "$_\n";
			$count++;
		}
		elsif (/^[A-Z]+\^[A-z]/ && length $_ < 9) {
			s/\^/_P/;
			print OUT "$_\n";
			$count++;
		}
		elsif (/^[A-Z]+\^/ && length $_ < 10) {
			s/\^.*//;
		}	
		elsif (length $_  < 10 && /[A-Z]/) {
			open ERROR, ">>$exchange.errors";
			print ERROR "$_\n";
			close ERROR;
		}
	}
	print "$exchange: $count records saved\n";
	close IN;
}